# -*- coding: utf-8 -*-

import sys
import os
import logging
import contextlib
import click
import subprocess
from jinja2 import Environment, FileSystemLoader


logger = logging.getLogger(__name__)
current_dir = os.path.dirname(os.path.abspath(__file__))
template_dir = os.path.join(current_dir, 'helloworld')

cwd = os.getcwd()


@contextlib.contextmanager
def cd(dir):
    os.chdir(dir)
    yield dir
    os.chdir(cwd)


def mkdir_p(name, strict=False):
    if os.path.exists(name) and os.path.isdir(name):
        return
    try:
        os.mkdir(name)
    except OSError:
        logger.warning("cannot mkdir %s", name)
        if strict:
            sys.exit(1)

jinja_env = Environment(loader=FileSystemLoader(template_dir))


def render_template(template, filename=None, folder='.', mapping=None):
    """Render ``template`` to ``filename`` with ``mapping``.
    :param template: template file name, e.g. 'settings.py.jinja'
    :param filename: filename to store as, e.g. 'settings.py'
    :param folder: target file folder, e.g. '.'
    :param mapping: key value mappings to render with
    """
    if mapping is None:
        mapping = {}

    if filename is None:
        if not template.endswith('.jinja'):
            raise ValueError("Filename extension '.jinja' required")
        filename = template[:-len('.jinja')]

    path = os.path.join(folder, filename)
    content = jinja_env.get_template(template).render(**mapping)

    if os.path.exists(path):
        logger.warn("%s already exists, skipping...", os.path.normpath(path))
        return

    mkdir_p(folder)

    logger.info("Rendering %s...", os.path.normpath(path))

    with open(path, 'w') as f:
        f.write(content.encode('utf8'))

    return path


def validate_app_id(ctx, argument, value):
    if len(value.split('.')) != 2:
        raise click.BadParameter("AppId should be like 'team.proj', "
                                 "with 2 elements")

    if not all(map(unicode.islower, value.split('.'))):
        raise click.BadParameter("AppId should be like 'team.proj', all "
                                 "in lower case")
    return value


def validate_service_name(ctx, argument, value):
    if not value:
        raise click.BadParameter("Service name should not be empty")

    if not value.isalpha():
        raise click.BadParameter("Service name should be all letters[A-Za-z]")

    if not value[0].isupper():
        raise click.BadParameter("Service name should start with [A-Z]")
    return value


@click.command()
@click.option('--app-id', prompt="Applicattion id (e.g. arch.note)",
              callback=validate_app_id, help="App ID like 'team.proj'")
@click.option('--service-name', prompt="Service name (pascal style)",
              callback=validate_service_name,
              help="Service name like 'Note'")
def bootstrap(app_id, service_name):
    from rest_arch import __version__
    app_slug = package_name = app_id.split('.')[1]
    mapping = dict(app_id=app_id,
                   service=service_name,
                   package=package_name,
                   app_slug=app_slug,
                   arch_version=__version__)
    logger.info("Createing new project: %s", app_id)

    mkdir_p(app_id)

    with cd(app_id):
        render_template('app.py.jinja', mapping=mapping,
                        folder=package_name)
        render_template('models.py.jinja', mapping=mapping,
                        folder=package_name)
        render_template('settings.py.jinja', mapping=mapping,
                        folder=package_name)
        render_template('requirements.txt.jinja', mapping=mapping)
        render_template('app.yaml.jinja', mapping=mapping)
        render_template('__init__.py.jinja', mapping=mapping,
                        folder=package_name)

        subprocess.check_call(['git', 'init'])
        logger.info("Bootstrap completed.")
        logger.info("Run `skt serve` in %s/ "
                    "to start server.", app_id)


if __name__ == '__main__':
    # pylint: disable=E1120
    bootstrap()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
  REST_ARCH
  ~~~~~~~

  RESTful Framework Based On Gunicorn
"""

version_info = (0, 1, 1, 0)
__version__ = ".".join([str(v) for v in version_info])



# -*- coding: utf-8 -*-

import logging
import contextlib
from .skt.config import load_app_config
from .ctx import g


def obj2str(obj):
    """Try to convert an object to string format."""
    if isinstance(obj, unicode):
        return obj.encode('utf8')
    elif isinstance(obj, (str, int, float, bool)):
        return str(obj)
    return repr(obj)


@contextlib.contextmanager
def logging_meta(**kwargs):
    """Logging with key-value meta, e.g.
    ::

        with logging_meta(key=val) as ctx:
            # do logging
    """
    conflicts = {}
    meta = g.logging_meta

    # if has conflicts, record the original kvs
    for key in kwargs:
        if key in meta:
            conflicts[key] = meta[key]

    try:
        meta.update(kwargs)
        yield meta
    finally:
        # recovery the original conflict kvs
        for key in kwargs:
            meta.pop(key)
        meta.update(conflicts)


class RestFormatter(logging.Formatter):
    """
    Logging formmater with meta (kv pairs) support.

    Skeletal structure::

        [required_meta] extra_meta ## body

    Detail structure::

        datetime level name[pid]: [app_id rpc_id request_id] k => v .. ## msg

    """

    def _format(self, msg):
        # required meta
        app_id = load_app_config().app_id
        request_id = g.get_call_meta('request_id')
        rpc_id = g.get_call_meta('seq')
        required_meta = '[{0} {1} {2}]'.format(app_id, rpc_id, request_id)

        # extra_meta
        metas = [required_meta]
        if g.logging_meta:
            for item in g.logging_meta.iteritems():
                k, v = map(obj2str, item)
                extra_meta = '%s => %s' % (k, v)
                metas.append(extra_meta)

        # meta
        meta = ' '.join(metas)
        # body
        body = obj2str(msg)
        # join msg
        return ' ## '.join([meta, body])

    def format(self, record):
        record.msg = self._format(record.msg)
        return super(RestFormatter, self).format(record)



# -*- coding: utf-8 -*-

import logging

from .consts import ENV_DEV


def setup_logger_cls():
    pass


def _gen_console_logging_config(logger_name):
    return {
        'version': 1,
        'disable_existing_loggers': False,
        'root': {
            'handlers': ['console'],
            'level': 'INFO',
        },
        'loggers': {
            logger_name: {
                'handlers': ['console'],
                'propagate': False,
                'level': 'INFO',
            },
        },
        'handlers': {
            'console': {
                'level': 'DEBUG',
                'class': 'logging.StreamHandler',
                'formatter': 'console',
            },
        },
        'formatters': {
            'console': {
                '()': 'rest_arch.log.RestFormatter',
                'format': ('%(asctime)s %(levelname)-6s '
                           '%(name)s[%(process)d] %(message)s'),
            },
        },
    }


def _gen_syslog_logging_config(logger_name):
    """Generate logging dict config for production env, if
    ``settings.SENTRY_DSN`` is provided, sentry logging handler will
    be enabled. """
    from .config import load_app_config
    cfg = load_app_config()
    return {
        'version': 1,
        'disable_existing_loggers': False,
        'root': {
            'handlers': ['syslog', 'syslog_udp'],
            'level': 'INFO',
        },
        'loggers': {
            '{}.rest_arch.sql_logger'.format(cfg.app_slug): {
                'handlers': ['syslog_udp'],
                'propagate': False,
                'level': 'INFO',
            },
            logger_name: {
                'handlers': ['syslog'],
                'propagate': False,
                'level': 'INFO',
            },
            'gunicorn.{0}'.format(cfg.app_name): {
                'handlers': ['syslog'],
                'propagate': False,
                'level': 'INFO',
            },
            'gunicorn.{0}.wsgi.access'.format(cfg.app_name): {
                'handlers': ['syslog'],
                'propagate': False,
                'level': 'INFO',
            },
            'gunicorn.{0}.wsgi.error'.format(cfg.app_name): {
                'handlers': ['syslog'],
                'propagate': False,
                'level': 'ERROR',
            },
            '{0}.rest_arch.async'.format(cfg.app_name): {
                'handlers': ['syslog'],
                'propagate': False,
                'level': 'INFO',
            },
        },
        'handlers': {
            'syslog_udp': {
                'level': 'INFO',
                'class': 'logging.handlers.SysLogHandler',
                'address': ('localhost', 514),
                'facility': 'local6',
                'formatter': 'syslog',
            },
            'syslog': {
                'level': 'INFO',
                'class': 'logging.handlers.SysLogHandler',
                'address': '/dev/log',
                'facility': 'local6',
                'formatter': 'syslog',
            },
        },
        'formatters': {
            'syslog': {
                '()': 'rest_arch.log.RestFormatter',
                'format': '%(name)s[%(process)d]: %(message)s',
            },
        },
    }


def gen_logging_dictconfig(logger_name, env):
    if env == ENV_DEV:
        conf = _gen_console_logging_config(logger_name)
    else:
        conf = _gen_syslog_logging_config(logger_name)
    return conf


def setup_loggers(logger_name, env=ENV_DEV):
    setup_logger_cls()
    conf = gen_logging_dictconfig(logger_name, env)
    logging.config.dictConfig(conf)
